package com.persagy.service.impl;

import cn.hutool.core.date.DateUtil;
import com.persagy.entity.ExpireAlarmMessage;
import com.persagy.job.AlarmExpireJob;
import com.persagy.job.ExpireAlarmQueue;
import com.persagy.service.AlarmQuartzService;
import com.persagy.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.*;

import static org.quartz.SimpleScheduleBuilder.simpleSchedule;

/**
 * @description:报警定时任务服务
 * @author:LuoGuanhyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/9/18 001820:11
 * @version:V1.0
 **/
@Slf4j
@Service
public class AlarmQuartzServiceImpl implements AlarmQuartzService {

    @Autowired
    @Qualifier("quartzScheduler")
    Scheduler quartzScheduler;

    @Override
    public void initAlarmMessagehandler() {
        try {
            System.out.println("--init--");
            //启动后先休息20秒，这样报警netty  websocket  quartz都启动了
            Thread.sleep(20000);
            while (true){
                ExpireAlarmMessage expireAlarmMessage = ExpireAlarmQueue.getExpireAlarmMessageQueue().consume();
                log.info("剩余过期消息总数:{}",ExpireAlarmQueue.getExpireAlarmMessageQueue().size());
                if("1".equals(expireAlarmMessage.getType())){
                    addExpireJob(expireAlarmMessage.getStartTime(),expireAlarmMessage.getJobName(),expireAlarmMessage.getJobGroupName(),expireAlarmMessage.getJobDataMap());
                }else if("2".equals(expireAlarmMessage.getType())){
                    deleteExpireJob(expireAlarmMessage.getJobName(),expireAlarmMessage.getJobGroupName());
                }
            }
        } catch (Exception e) {
            log.error("报警过期消息队列消费失败",e);
        }
    }
    /**
     * 添加定时器
     * @param startTime 添加报警恢复定时任务
     * @param jobName   job名称
     * @param jobGroupName  job分组
     * @param jobDataMap  触发器的数据（任务相关的数据都放在这里）
     * @return 
     * @throws SchedulerException
     */
    @Override
    public String addExpireJob(Date startTime, String jobName, String jobGroupName, JobDataMap jobDataMap) throws SchedulerException {
        if(startTime.before(new Date())){
            log.error("执行时间【{}】为历史时间，继续添加", DateUtils.formatDate(startTime));
        }
        JobKey jobKey = new JobKey(jobName, jobGroupName);
        JobDetail jobDetail = JobBuilder.newJob(AlarmExpireJob.class).withIdentity(jobName, jobGroupName).requestRecovery().build();
        Trigger trigger = TriggerBuilder.newTrigger().startAt(startTime)
                .withIdentity("trigger_" + jobName , jobName)
                .usingJobData(jobDataMap)
                .withSchedule(simpleSchedule().withMisfireHandlingInstructionIgnoreMisfires()).build();
        //已经存在的job实例和触发器自动覆盖 job实 例 唯 一标识：
        HashSet<Trigger> triggerSet = new HashSet<Trigger>();
        Date fireTime = trigger.getFireTimeAfter(DateUtil.offsetMinute(startTime,-1).toJdkDate());
        if(fireTime != null){
            log.info("执行时间为:【{}】，设置时间为【{}】",DateUtils.formatDate(fireTime),DateUtils.formatDate(startTime));
        }else {
            log.info("执行时间为 空!");
        }
        triggerSet.add(trigger);
        quartzScheduler.scheduleJob(jobDetail, triggerSet, true);
        return "success";
    }


    /**
     * @description:删除定时任务
     * @exception:
     * @author: LuoGuangyi
     * @company: Persagy Technology Co.,Ltd
     * @param jobName:job名称
     * @param jobGroupName:job分组名称
     * @return: java.lang.String
     * @since: 2020/09/29 9:23
     * @version: V1.0
     */
    @Override
    public synchronized String deleteExpireJob(String jobName, String jobGroupName) throws SchedulerException {
        JobKey jobKey = new JobKey(jobName, jobGroupName);
        quartzScheduler.deleteJob(jobKey);
        return "success!";
    }


}
